@@ -137,6 +137,7 @@ class TestOutput(object):
 
         [self.attribute(attr) for attr in self.test.attributes.values()]
         [self.requirement(req) for req in self.test.requirements.values()]
+        [self.specification(spec) for spec in self.test.specifications.values()]
         [self.argument(arg) for arg in self.test.args.values()]
         [self.tag(Tag(tag)) for tag in self.test.tags]
         [self.example(ExamplesRow(row._idx, row._fields, [str(f) for f in row], row._row_format)) for row in self.test.examples]
@@ -147,12 +148,19 @@ class TestOutput(object):
 
     def attribute(self, attribute, object_type=MessageObjectType.TEST):
         msg = object_fields(attribute, "attribute")
+        value = msg["attribute_value"]
+        if value is not None:
+            msg["attribute_value"] = str_or_repr(value)
         self.message(Message.ATTRIBUTE, msg, object_type=object_type)
 
     def requirement(self, requirement, object_type=MessageObjectType.TEST):
         msg = object_fields(requirement, "requirement")
         self.message(Message.REQUIREMENT, msg, object_type=object_type)
 
+    def specification(self, specification, object_type=MessageObjectType.TEST):
+        msg = object_fields(specification, "specification")
+        self.message(Message.SPECIFICATION, msg, object_type=object_type)
+
     def argument(self, argument, object_type=MessageObjectType.TEST):
         msg = object_fields(argument, "argument")
         value = msg["argument_value"]
@@ -414,7 +422,8 @@ class LogWriter(object):
                 self.fd.write(compress(b"".join(self.buffer)))
                 self.fd.flush()
                 del self.buffer[:]
-            if not final:
+
+            if not final and threading.main_thread().is_alive():
                 self.timer = threading.Timer(self.auto_flush_interval, self.flush, (True,))
                 self.timer.start()
 
