@@ -33,7 +33,8 @@ from .flags import Flags, SKIP, TE, FAIL_NOT_COUNTED, ERROR_NOT_COUNTED, NULL_NO
 from .flags import CFLAGS, PAUSE_BEFORE, PAUSE_AFTER
 from .testtype import TestType, TestSubType
 from .objects import get, Null, OK, Fail, Skip, Error, PassResults, Argument, Attribute, Requirement, ArgumentParser
-from .objects import RepeatTest, ExamplesTable
+from .objects import RepeatTest, ExamplesTable, Specification
+from .objects import Secret
 from .constants import name_sep, id_sep
 from .io import TestIO, LogWriter
 from .name import join, depth, match, absname, split, isabs
@@ -42,7 +43,8 @@ from .init import init
 from .cli.arg.parser import ArgumentParser as ArgumentParserClass
 from .cli.arg.common import epilog as common_epilog
 from .cli.arg.exit import ExitWithError, ExitException
-from .cli.arg.type import key_value as key_value_type, repeat as repeat_type, tags_filter as tags_filter_type, logfile as logfile_type
+from .cli.arg.type import key_value as key_value_type, repeat as repeat_type, tags_filter as tags_filter_type
+from .cli.arg.type import logfile as logfile_type, rsa_private_key_pem_file as rsa_private_key_pem_file_type
 from .cli.text import danger, warning
 from .exceptions import exception as get_exception
 from .filters import The, TheTags
@@ -161,6 +163,7 @@ class TestBase(object):
     tags = set()
     attributes = []
     requirements = []
+    specifications = []
     examples = None
     name = None
     description = None
@@ -172,12 +175,12 @@ class TestBase(object):
     subtype = None
 
     def __init__(self, name=None, flags=None, cflags=None, type=None, subtype=None,
-                 uid=None, tags=None, attributes=None, requirements=None,
+                 uid=None, tags=None, attributes=None, requirements=None, specifications=None,
                  examples=None, description=None, parent=None,
                  xfails=None, xflags=None, only=None, skip=None,
                  start=None, end=None, only_tags=None, skip_tags=None,
                  args=None, id=None, node=None, map=None, context=None,
-                 repeat=None):
+                 repeat=None, private_key=None):
 
         self.lock = threading.Lock()
 
@@ -204,8 +207,9 @@ class TestBase(object):
         self.subtype = get(subtype, self.subtype)
         self.context = get(context, current_test.context if current_test and self.type < TestType.Iteration else (Context(current_test.context if current_test else None)))
         self.tags = tags
+        self.specifications = {s.name: s for s in [Specification(*r) for r in get(specifications, list(self.specifications))]}
         self.requirements = {r.name: r for r in [Requirement(*r) for r in get(requirements, list(self.requirements))]}
-        self.attributes =  {a.name: a for a in [Attribute(*a) for a in get(attributes, list(self.attributes))]}
+        self.attributes = {a.name: a for a in [Attribute(*a) for a in get(attributes, list(self.attributes))]}
         self.args = {k: Argument(k, v) for k,v in get(args, {}).items()}
         self.description = get(description, self.description)
         self.examples = get(examples, get(self.examples, ExamplesTable()))
@@ -214,6 +218,8 @@ class TestBase(object):
         self.result = Null(test=self.name)
         if flags is not None:
             self.flags = Flags(flags)
+        if self.subtype in [TestSubType.Given, TestSubType.Finally]:
+            self.flags |= MANDATORY
         self.cflags = Flags(cflags) | (self.flags & CFLAGS)
         self.uid = get(uid, self.uid)
         if self.uid is not None:
@@ -227,6 +233,7 @@ class TestBase(object):
         self.only_tags = get(only_tags, None)
         self.skip_tags = get(skip_tags, None)
         self.repeat = get(repeat, None)
+        self.private_key = get(private_key, None)
         self.caller_test = None
 
     @classmethod
@@ -270,10 +277,18 @@ class TestBase(object):
 
     def _enter(self):
         self.io = TestIO(self)
+
         if top() is self:
+            self._init = init()
             self.io.output.protocol()
             self.io.output.version()
 
+            self.attributes.update({
+                    arg.name: Attribute(name=arg.name, value=arg.value, type=arg.type, group=arg.group, uid=arg.uid)
+                    for arg in self.args.values()
+                })
+            self.args = {}
+
         self.io.output.test_message()
 
         if self.flags & PAUSE_BEFORE and not self.flags & SKIP:
@@ -284,14 +299,13 @@ class TestBase(object):
 
         if self.flags & SKIP:
             raise Skip("skip flag set", test=self.name)
-        else:
-            if top() is self:
-                self._init = init()
-            return self
+
+        return self
 
     def _exit(self, exc_type, exc_value, exc_traceback):
         if not self.io:
             return False
+
         if top() is self and not self._init:
             return False
 
@@ -473,6 +487,9 @@ def cli_argparser(kwargs, argparser=None):
     parser.add_argument("--individually", dest="_individually", action="store_true", default=False,
                         help="if --rerun is specified then rerun tests in the --reference log file individually.")
 
+    parser.add_argument("--private-key", dest="_private_key", metavar="file", type=rsa_private_key_pem_file_type,
+        help="RSA private key PEM file that can be used to encrypt secrets.")
+
     if database_module:
         database_module.argparser(parser)
 
@@ -586,7 +603,15 @@ def parse_cli_args(kwargs, parser):
             kwargs["tags"] = {value for value in args.pop("_tags")}
 
         if args.get("_attrs"):
-            kwargs["attributes"] = [Attribute(item.key, item.value) for item in args.pop("_attrs")]
+            if kwargs.get("attributes", None) is None:
+                kwargs["attributes"] = []
+            kwargs["attributes"] += [Attribute(item.key, item.value) for item in args.pop("_attrs")]
+            for attr in kwargs["attributes"]:
+                if args.get(attr.name, None):
+                    raise AttributeError(f"use test argument '--{attr.name}' instead of '--attr {attr.name}=<value>'")
+
+        if args.get("_private_key"):
+            kwargs["private_key"] = args.pop("_private_key")
 
         if args.get("_repeat"):
             repeat = []
@@ -673,6 +698,16 @@ def parse_cli_args(kwargs, parser):
                 else:
                     kwargs["only"].append(The(join(rerun_test.name, "*")))
 
+        if args.get("func"):
+            func = args.pop("func")
+            func(args, kwargs)
+
+        if kwargs.get("private_key"):
+            private_key = kwargs.get("private_key")
+            for name, value in args.items():
+                if isinstance(value, Secret):
+                    value(public_key=private_key.pubkey)
+
     except (ExitException, KeyboardInterrupt, Exception) as exc:
         if not debug_processed or settings.debug:
             sys.stderr.write(warning(get_exception(), eol='\n'))
@@ -784,8 +819,8 @@ class TestDefinition(object):
             argparser = kwargs.pop("argparser", None)
             parent = kwargs.pop("parent", None) or current()
             keep_type = kwargs.pop("keep_type", None)
-            format_name = kwargs.pop("format_name", True)
-            format_description = kwargs.pop("format_description", True)
+            format_name = kwargs.pop("format_name", False)
+            format_description = kwargs.pop("format_description", False)
 
             if not top():
                 cli_args = parse_cli_args(self.kwargs, cli_argparser(self.kwargs, argparser if not isinstance(argparser, ArgumentParser) else argparser.value))
@@ -948,6 +983,7 @@ class TestDefinition(object):
         except (KeyboardInterrupt, Exception) as exc:
             if not self.test.io:
                 raise
+
             frame = inspect.currentframe().f_back
             self._with_block_frame = (frame, frame.f_lasti, frame.f_lineno)
             self.trace = sys.gettrace()
@@ -1076,6 +1112,7 @@ class TestDefinition(object):
 
     def __exit__(self, exception_type, exception_value, exception_traceback):
         frame = inspect.currentframe().f_back
+        co_filename_filter = "testflows/_core"
 
         def make_complete_traceback(exception_traceback, frame):
             tb = namedtuple('tb', ('tb_frame', 'tb_lasti', 'tb_lineno', 'tb_next'))
@@ -1083,7 +1120,7 @@ class TestDefinition(object):
             def walk_frame(frame, tb_next=None):
                 if frame is None:
                     return tb_next
-                if not settings.debug and frame.f_code.co_filename.endswith("testflows/_core/test.py"):
+                if not settings.debug and co_filename_filter in frame.f_code.co_filename:
                     tb_next = tb_next
                 else:
                     tb_next = tb(frame, frame.f_lasti, frame.f_lineno, tb_next)
@@ -1094,7 +1131,7 @@ class TestDefinition(object):
                 if tb_frame.tb_next:
                     tb_next = walk_tb(tb_frame.tb_next)
 
-                if tb_frame and not settings.debug and tb_frame.tb_frame.f_code.co_filename.endswith("testflows/_core/test.py"):
+                if tb_frame and not settings.debug and co_filename_filter in tb_frame.tb_frame.f_code.co_filename:
                     return tb_next
                 else:
                     return tb(tb_frame.tb_frame, tb_frame.tb_lasti, tb_frame.tb_lineno, tb_next)
@@ -1206,6 +1243,10 @@ class TestDefinition(object):
                 raise
 
         if not self.parent:
+            if not test__exit__:
+                sys.stderr.write(warning(get_exception(exception_type, exception_value, exception_traceback), eol='\n'))
+                sys.stderr.write(danger("error: " + str(exception_value).strip()))
+                sys.exit(1)
             sys.exit(0 if self.test.result else 1)
 
         if isinstance(exception_value, KeyboardInterrupt):
