@@ -22,13 +22,15 @@ from testflows._core.flags import Flags
 from testflows._core.objects import OK, XOK, Fail, XFail, Skip, Error, XError, Null, XNull
 from testflows._core.objects import Name, Description, Uid, Tags, Args
 from testflows._core.objects import XFails, XFlags, Repeat, RepeatTest
-from testflows._core.objects import Attributes, Requirements, Examples, ArgumentParser
-from testflows._core.objects import Node, Tag, Argument, Attribute, Requirement, Metric, Value, Ticket
+from testflows._core.objects import Attributes, Requirements, Specifications, Examples, ArgumentParser
+from testflows._core.objects import Node, Tag, Argument, Attribute, Requirement, Specification, Metric, Value, Ticket
+from testflows._core.objects import Secret
 from testflows._core.baseobject import Table
 from testflows._core.filters import The, TheTags
 from testflows._core.funcs import top, current, previous, load, append_path
-from testflows._core.funcs import main, args
+from testflows._core.funcs import main, args, private_key
 from testflows._core.funcs import metric, ticket, value, note, debug, trace
+from testflows._core.funcs import attribute, requirement, tag
 from testflows._core.funcs import message, exception, ok, fail, skip, err
 from testflows._core.funcs import null, xok, xfail, xerr, xnull, pause, getsattr
 from testflows._core.funcs import current_dir, current_module, load_module
@@ -42,3 +44,4 @@ from testflows._core import __author__, __version__, __license__
 from testflows._core import threading
 
 import testflows._core.utils as utils
+import testflows._core.contrib.rsa as rsa
\ No newline at end of file
